"use strict";
/// <reference path="../../../node_modules/@types/jquery/JQueryStatic.d.ts" />
Object.defineProperty(exports, "__esModule", { value: true });
exports.JQueryClient = exports.DEFAULT_ERROR_MESSAGE = void 0;
const tslib_1 = require("tslib");
const http_client_base_1 = require("@odata2ts/http-client-base");
const AjaxRequestConfig_1 = require("./AjaxRequestConfig");
const JQueryClientError_1 = require("./JQueryClientError");
exports.DEFAULT_ERROR_MESSAGE = "No error message!";
class JQueryClient extends http_client_base_1.BaseHttpClient {
    constructor(jquery, config, clientOptions) {
        super(clientOptions);
        this.client = jquery;
        this.config = (0, AjaxRequestConfig_1.getDefaultConfig)(config);
    }
    mapHeaders(jqXhr) {
        return jqXhr
            .getAllResponseHeaders()
            .trim()
            .split(/[\r\n]+/)
            .reduce((collector, line) => {
            const parts = line.split(": ");
            const header = parts.shift();
            const value = parts.join(": ");
            if (header) {
                collector[header.toLowerCase()] = value;
            }
            return collector;
        }, {});
    }
    executeRequest(method, url, data, requestConfig, internalConfig = {}) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const withInternalConfig = (0, AjaxRequestConfig_1.mergeAjaxConfig)({ headers: internalConfig.headers }, requestConfig);
            const _a = (0, AjaxRequestConfig_1.mergeAjaxConfig)(this.config, withInternalConfig), { params } = _a, mergedConfig = tslib_1.__rest(_a, ["params"]);
            mergedConfig.method = method;
            mergedConfig.data = JSON.stringify(data);
            mergedConfig.url = url;
            if (params && Object.values(params).length) {
                mergedConfig.url +=
                    (url.match(/\?/) ? "&" : "?") +
                        // @ts-ignore
                        new URLSearchParams(params).toString();
            }
            // the actual request
            return new Promise((resolve, reject) => {
                this.client.ajax(Object.assign(Object.assign({}, mergedConfig), { success: (response, textStatus, jqXHR) => {
                        resolve({
                            status: jqXHR.status,
                            statusText: jqXHR.statusText,
                            headers: this.mapHeaders(jqXHR),
                            data: response,
                        });
                    }, error: (jqXHR, textStatus, thrownError) => {
                        var _a;
                        const responseMessage = this.retrieveErrorMessage(jqXHR === null || jqXHR === void 0 ? void 0 : jqXHR.responseJSON);
                        const failMsg = (_a = responseMessage !== null && responseMessage !== void 0 ? responseMessage : thrownError) !== null && _a !== void 0 ? _a : exports.DEFAULT_ERROR_MESSAGE;
                        const errorMessage = responseMessage ? "OData server responded with error: " + responseMessage : failMsg;
                        const responseHeaders = this.mapHeaders(jqXHR);
                        reject(new JQueryClientError_1.JQueryClientError(errorMessage, jqXHR.status, responseHeaders, new Error(failMsg), jqXHR));
                    } }));
            });
        });
    }
}
exports.JQueryClient = JQueryClient;
//# sourceMappingURL=JQueryClient.js.map